using System;
using System.Runtime.InteropServices;

namespace CnBlogs.Helper.hBifTs
{
	#region Struct info
	[StructLayout(LayoutKind.Sequential)]
	public struct ACL
	{
		public byte AclRevision;
		public byte Sbz1;
		public uint AclSize;
		public uint AceCount;
		public uint Sbz2;
	}

	[StructLayout(LayoutKind.Sequential)]
	public struct SECURITY_ATTRIBUTES
	{
		public uint nLength;
		public IntPtr lpSecurityDescriptor; // point to SECURITY_DESCRIPTOR
		public int bInheritHandle;
	}

	[StructLayout(LayoutKind.Sequential)]
	public struct SECURITY_DESCRIPTOR
	{
		public byte Revision;
		public byte Sbz1;
		public int Control;   // point to SECURITY_DESCRIPTOR_CONTROL  typedef WORD   SECURITY_DESCRIPTOR_CONTROL
		public IntPtr Owner;  // point to PSID   typedef PVOID PSID
		public IntPtr Group;  // point to PSID   typedef PVOID PSID
		public IntPtr Sacl;   // point to ACL
		public IntPtr Dacl;	  // point to ACL
	}
	#endregion

	#region SecurityStruct
	/// <summary>
	/// Summary description for SecuritySturct.
	/// </summary>
	public class SecurityStruct : IDisposable
	{
		#region DllImport
		private const int SECURITY_DESCRIPTOR_REVISION = 1;

		[DllImport("advapi32.dll", SetLastError=true)]
		static extern bool InitializeSecurityDescriptor(IntPtr pSecurityDescriptor, uint dwRevision);

		[DllImport("advapi32.dll", SetLastError=true)]
		static extern bool SetSecurityDescriptorDacl( IntPtr pSecurityDescriptor,int bDaclPresent,IntPtr pDacl,int bDaclDefaulted);
		#endregion

		#region Private variable & Propery
		private IntPtr lpSecurityDescriptor;
		private IntPtr pSa;
		#endregion

		#region Constructor & Dispose
		public SecurityStruct()
		{
		}
		
		public void Close()
		{
			if( lpSecurityDescriptor != IntPtr.Zero)
			{
				Marshal.FreeCoTaskMem( lpSecurityDescriptor);
				lpSecurityDescriptor = IntPtr.Zero;
			}
			if( pSa != IntPtr.Zero)
			{
				Marshal.FreeCoTaskMem( pSa);
				pSa = IntPtr.Zero;
			}
		}

		public void Dispose()
		{
			Close();		
		}
		#endregion

		#region Common Function
		public IntPtr AnonymousSA()
		{
			return AnonymousSA(false);
		}

		public IntPtr AnonymousSA(bool bInheritHandle)
		{
			Close();

			SECURITY_ATTRIBUTES sa = new SECURITY_ATTRIBUTES();
			SECURITY_DESCRIPTOR sd = new SECURITY_DESCRIPTOR();

			lpSecurityDescriptor = Marshal.AllocCoTaskMem( Marshal.SizeOf( sd));
			Marshal.StructureToPtr( sd,lpSecurityDescriptor,false);

			if( ! InitializeSecurityDescriptor( lpSecurityDescriptor ,SECURITY_DESCRIPTOR_REVISION))
			{
				throw new ApplicationException("InitializeSecurityDescriptor invoked fail");
			}

			if( ! SetSecurityDescriptorDacl( lpSecurityDescriptor ,1,IntPtr.Zero,0))
			{
				throw new ApplicationException("SetSecurityDescriptorDacl invoked fail");
			}

			sa.bInheritHandle = bInheritHandle?1:0;
			sa.nLength = (uint)Marshal.SizeOf( sa );

			sa.lpSecurityDescriptor = lpSecurityDescriptor;

			pSa = Marshal.AllocCoTaskMem( Marshal.SizeOf( sa));
			Marshal.StructureToPtr( sa,pSa,false);
            
			return pSa;
		}
		#endregion
	}
	#endregion
}
